import json
import os.path
from django import http

from .templatetags.jsform import get_errormessage_id

def d(text):
    return json.dumps(text)

def js_response(content):
    return http.HttpResponse(content, content_type='text/javascript')

def reload():
    return js_response('location.reload()')

def go_back():
    '''
        This is our recommend response for "Edit" form pages.
        Use with some means of "stale page detection" to trigger the previous page to reload.
    '''
    return js_response('history.back()')
def go_to(url):
    '''
        Useful if you have an in-page "Add" form on a list page, and want to navigate to detail page for the created object upon form completion.
    '''
    return js_response(f'location = {d(url)}')
def replace_location(url):
    '''
        We officially endorse NavTricks.js
        https://github.com/quadrant-newmedia/NavTricks
    '''
    return js_response(f'''
        if (window.NavTricks) {{
            NavTricks.replaceCurrentPage({d(url)});
        }}
        else {{
            location.replace({d(url)});
        }}
    ''')

def alert(message):
    '''
        Useful for error messages that aren't really typical form error messages, or for actions that are triggered by a simple button push (where there is no corresponding container for error messages).
    '''
    return js_response(f'alert({d(message)})')

def _get_script_content(script_name):
    with open(os.path.join(os.path.dirname(__file__), 'js_helpers', script_name)) as f :
        return f.read()

'''
    Form manipulation helpers.

    Note that we generally recommend using jsform_elementmerge.js and simply returning html pages to be merged into the current document.

    In particular circumstances, you may find it easier to mutate the form directly, and you can use these.

    Note that all of these functions return true, to allow further form submissions.
'''
def reset_form_inputs():
    '''
        Resets form and returns true, to allow further submissions.
    '''
    return js_response(f'form.reset(); return true')

def clear_form_errors():
    '''
        Requires django
        Resets form and returns true, to allow further submissions.
    '''
    return js_response(f'''
        {_get_script_content("clear_form_errors.js")};
        clear_form_errors(form); 
        return true
    ''')
def set_form_errors(form):
    return set_raw_form_errors(
        form.non_field_errors(), 
        {get_errormessage_id(field): field.errors for field in form}
    )
def set_raw_form_errors(form_errors, error_map):
    return js_response(f'''
        {_get_script_content("set_form_errors.js")};
        set_form_errors(form, {d(form_errors)}, {d(error_map)});
        return true
    ''')
def reset_form():js_response(f'''
        {_get_script_content("js_helpers/clear_form_errors.js")};
        clear_form_errors(form); 
        form.reset();
        return true
    ''')


from django.test import TestCase, override_settings

# Sample - you can override any settings you require for your tests
# @override_settings(ROOT_URLCONF='django_jsform.tests.urls')
class MyTestCase(TestCase):
    pass
    # Samples:
    # def test_normal_path_before_dynamic_returns_correct_response(self):
    #     r = self.client.get('/path_before/')
    #     self.assertEqual(r.content, b'path before')
    # def test_normal_path_after_dynamic_returns_correct_response(self):
    #     r = self.client.get('/path_after/')
    #     self.assertEqual(r.content, b'path after')

    # def test_dynamic_path_returns_expected_response(self):
    #     r = self.client.get('/bar/')
    #     self.assertEqual(r.content, b'bar from dynamic')

    # def test_dynamic_path_func_receives_correct_args(self):
    #     r = self.client.get('/baz/')
    #     self.assertEqual(r.status_code, 200)

    # def test_included_dynamic_path_func_receives_correct_args(self):
    #     r = self.client.get('/included/baz/')
    #     self.assertEqual(r.status_code, 200)

    # def test_path_before_can_be_reversed(self):
    #     self.assertEqual(reverse('path_before_name'), '/path_before/')
    # def test_path_after_with_params_can_be_reversed(self):
    #     self.assertEqual(reverse('path_after_name', kwargs=dict(value='BAZ')), '/path_after/BAZ/')
